import IConfig from './Interfaces/IConfig';
declare class FluentMerge<T = any> {
    private target;
    private sources;
    private config;
    /**
     * Supplies a fluent merge instance with a target object to merge into and return.
     */
    to(target: T): FluentMerge;
    /**
     * Supplies a fluent merge instance with one or more source objects to merge from, in right to left order.
     */
    from(...sources: any[]): FluentMerge;
    /**
     * Supplies a fluent merge instance with a configuration object of one or more options.
     */
    with(options?: IConfig | true): FluentMerge;
    /**
     * Executes a fluent merge instance, merging all provided sources into the
     * target, as per any provided configuration, and returning a reference to
     * the target.
     */
    exec(): T;
}
export default FluentMerge;
